@@ -32,7 +32,11 @@ __all__ = [
 
 import atexit
 import errno
+from hashlib import md5
+from io import BytesIO
+from json import dumps
 import os
+import re
 import shutil
 import sys
 import tempfile
@@ -41,7 +45,6 @@ import tempfile
 from time import sleep
 from httplib2 import (
     Http,
-    proxy_info_from_environment,
     urlnorm,
     )
 try:
@@ -49,17 +52,6 @@ try:
 except ImportError:
     from httplib2 import ProxyInfo
     proxy_info_from_environment = ProxyInfo.from_environment
-try:
-    # Python 3 and sufficiently modern Python 2 versions.
-    from json import dumps
-except ImportError:
-    from simplejson import dumps
-
-try:
-    # Python 3.
-    from io import BytesIO
-except ImportError:
-    from cStringIO import StringIO as BytesIO
 
 try:
     # Python 3.
@@ -81,22 +73,28 @@ else:
     str_types = str
 
 
-# A drop-in replacement for httplib2's safename.
-from httplib2 import _md5, re_url_scheme, re_slash
+# A drop-in replacement for httplib2's safename.  Substantially borrowed
+# from httplib2, but its cache name format changed in 0.12.0 and we want to
+# stick with the previous version.
+
+re_url_scheme = re.compile(br'^\w+://')
+re_url_scheme_s = re.compile(r'^\w+://')
+re_slash = re.compile(br'[?/:|]+')
+
+
 def safename(filename):
     """Return a filename suitable for the cache.
 
     Strips dangerous and common characters to create a filename we
     can use to store the cache in.
     """
-
     try:
-        if isinstance(filename, unicode_type):
-            filename_match = filename.encode('utf-8')
+        if isinstance(filename, bytes):
+            filename_match = filename.decode('utf-8')
         else:
             filename_match = filename
 
-        if re_url_scheme.match(filename_match):
+        if re_url_scheme_s.match(filename_match):
             if isinstance(filename, bytes):
                 filename = filename.decode('utf-8')
                 filename = filename.encode('idna')
@@ -106,7 +104,7 @@ def safename(filename):
         pass
     if isinstance(filename, unicode_type):
         filename = filename.encode('utf-8')
-    filemd5 = _md5(filename).hexdigest()
+    filemd5 = md5(filename).hexdigest()
     filename = re_url_scheme.sub(b"", filename)
     filename = re_slash.sub(b",", filename)
 
@@ -127,7 +125,7 @@ def safename(filename):
     maximum_filename_length = RestfulHttp.maximum_cache_filename_length
     maximum_length_before_md5_sum = maximum_filename_length - 32 - 1
     if len(filename) > maximum_length_before_md5_sum:
-        filename=filename[:maximum_length_before_md5_sum]
+        filename = filename[:maximum_length_before_md5_sum]
     return ",".join((filename.decode('utf-8'), filemd5))
 
 
